@@ -9,6 +9,14 @@ from torch.nn.modules.utils import _pair
 from ..builder import PIPELINES
 
 
+def _combine_quadruple(a, b):
+    return (a[0] + a[2] * b[0], a[1] + a[3] * b[1], a[2] * b[2], a[3] * b[3])
+
+
+def _flip_quadruple(a):
+    return (1 - a[0] - a[2], a[1], a[2], a[3])
+
+
 def _init_lazy_if_proper(results, lazy):
     """Initialize lazy operation properly.
 
@@ -44,6 +52,108 @@ def _init_lazy_if_proper(results, lazy):
 
 
 @PIPELINES.register_module()
+class PoseCompact:
+    """Convert the coordinates of keypoints to make it more compact.
+    Specifically, it first find a tight bounding box that surrounds all joints
+    in each frame, then we expand the tight box by a given padding ratio. For
+    example, if 'padding == 0.25', then the expanded box has unchanged center,
+    and 1.25x width and height.
+
+    Required keys in results are "img_shape", "keypoint", add or modified keys
+    are "img_shape", "keypoint", "crop_quadruple".
+
+    Args:
+        padding (float): The padding size. Default: 0.25.
+        threshold (int): The threshold for the tight bounding box. If the width
+            or height of the tight bounding box is smaller than the threshold,
+            we do not perform the compact operation. Default: 10.
+        hw_ratio (float | tuple[float] | None): The hw_ratio of the expanded
+            box. Float indicates the specific ratio and tuple indicates a
+            ratio range. If set as None, it means there is no requirement on
+            hw_ratio. Default: None.
+        allow_imgpad (bool): Whether to allow expanding the box outside the
+            image to meet the hw_ratio requirement. Default: True.
+
+    Returns:
+        type: Description of returned object.
+    """
+
+    def __init__(self,
+                 padding=0.25,
+                 threshold=10,
+                 hw_ratio=None,
+                 allow_imgpad=True):
+
+        self.padding = padding
+        self.threshold = threshold
+        if hw_ratio is not None:
+            hw_ratio = _pair(hw_ratio)
+
+        self.hw_ratio = hw_ratio
+
+        self.allow_imgpad = allow_imgpad
+        assert self.padding >= 0
+
+    def __call__(self, results):
+        img_shape = results['img_shape']
+        h, w = img_shape
+        kp = results['keypoint']
+
+        # Make NaN zero
+        kp[np.isnan(kp)] = 0.
+        kp_x = kp[..., 0]
+        kp_y = kp[..., 1]
+
+        min_x = np.min(kp_x[kp_x != 0], initial=np.Inf)
+        min_y = np.min(kp_y[kp_y != 0], initial=np.Inf)
+        max_x = np.max(kp_x[kp_x != 0], initial=-np.Inf)
+        max_y = np.max(kp_y[kp_y != 0], initial=-np.Inf)
+
+        # The compact area is too small
+        if max_x - min_x < self.threshold or max_y - min_y < self.threshold:
+            return results
+
+        center = ((max_x + min_x) / 2, (max_y + min_y) / 2)
+        half_width = (max_x - min_x) / 2 * (1 + self.padding)
+        half_height = (max_y - min_y) / 2 * (1 + self.padding)
+
+        if self.hw_ratio is not None:
+            half_height = max(self.hw_ratio[0] * half_width, half_height)
+            half_width = max(1 / self.hw_ratio[1] * half_height, half_width)
+
+        min_x, max_x = center[0] - half_width, center[0] + half_width
+        min_y, max_y = center[1] - half_height, center[1] + half_height
+
+        # hot update
+        if not self.allow_imgpad:
+            min_x, min_y = int(max(0, min_x)), int(max(0, min_y))
+            max_x, max_y = int(min(w, max_x)), int(min(h, max_y))
+        else:
+            min_x, min_y = int(min_x), int(min_y)
+            max_x, max_y = int(max_x), int(max_y)
+
+        kp_x[kp_x != 0] -= min_x
+        kp_y[kp_y != 0] -= min_y
+
+        new_shape = (max_y - min_y, max_x - min_x)
+        results['img_shape'] = new_shape
+
+        # the order is x, y, w, h (in [0, 1]), a tuple
+        crop_quadruple = results.get('crop_quadruple', (0., 0., 1., 1.))
+        new_crop_quadruple = (min_x / w, min_y / h, (max_x - min_x) / w,
+                              (max_y - min_y) / h)
+        crop_quadruple = _combine_quadruple(crop_quadruple, new_crop_quadruple)
+        results['crop_quadruple'] = crop_quadruple
+        return results
+
+    def __repr__(self):
+        repr_str = (f'{self.__class__.__name__}(padding={self.padding}, '
+                    f'threshold={self.threshold}, '
+                    f'hw_ratio={self.hw_ratio}, '
+                    f'allow_imgpad={self.allow_imgpad})')
+        return repr_str
+
+
 class EntityBoxRescale:
 
     def __init__(self, scale_factor):
@@ -428,9 +538,10 @@ class RandomScale:
 class RandomCrop:
     """Vanilla square random crop that specifics the output size.
 
-    Required keys in results are "imgs" and "img_shape", added or
-    modified keys are "imgs", "lazy"; Required keys in "lazy" are "flip",
-    "crop_bbox", added or modified key is "crop_bbox".
+    Required keys in results are "img_shape", "keypoint" (optional), "imgs"
+    (optional), added or modified keys are "keypoint", "imgs", "lazy"; Required
+    keys in "lazy" are "flip", "crop_bbox", added or modified key is
+    "crop_bbox".
 
     Args:
         size (int): The output size of the images.
@@ -443,6 +554,13 @@ class RandomCrop:
         self.size = size
         self.lazy = lazy
 
+    def _crop_kps(self, kps, crop_bbox):
+        return kps - crop_bbox[:2]
+
+    def _crop_imgs(self, imgs, crop_bbox):
+        x1, y1, x2, y2 = crop_bbox
+        return [img[y1:y2, x1:x2] for img in imgs]
+
     def _box_crop(self, box, crop_bbox):
         """Crop the bounding boxes according to the crop_bbox.
 
@@ -482,6 +600,9 @@ class RandomCrop:
                 to the next transform in pipeline.
         """
         _init_lazy_if_proper(results, self.lazy)
+        if 'keypoint' in results:
+            assert not self.lazy, ('Keypoint Augmentations are not compatible '
+                                   'with lazy == True')
 
         img_h, img_w = results['img_shape']
         assert self.size <= img_h and self.size <= img_w
@@ -521,10 +642,11 @@ class RandomCrop:
         results['img_shape'] = (new_h, new_w)
 
         if not self.lazy:
-            results['imgs'] = [
-                img[y_offset:y_offset + new_h, x_offset:x_offset + new_w]
-                for img in results['imgs']
-            ]
+            if 'keypoint' in results:
+                results['keypoint'] = self._crop_kps(results['keypoint'],
+                                                     crop_bbox)
+            if 'imgs' in results:
+                results['imgs'] = self._crop_imgs(results['imgs'], crop_bbox)
         else:
             lazyop = results['lazy']
             if lazyop['flip']:
@@ -559,9 +681,10 @@ class RandomCrop:
 class RandomResizedCrop(RandomCrop):
     """Random crop that specifics the area and height-weight ratio range.
 
-    Required keys in results are "imgs", "img_shape", "crop_bbox" and "lazy",
-    added or modified keys are "imgs", "crop_bbox" and "lazy"; Required keys
-    in "lazy" are "flip", "crop_bbox", added or modified key is "crop_bbox".
+    Required keys in results are "img_shape", "crop_bbox", "imgs" (optional),
+    "keypoint" (optional), added or modified keys are "imgs", "keypoint",
+    "crop_bbox" and "lazy"; Required keys in "lazy" are "flip", "crop_bbox",
+    added or modified key is "crop_bbox".
 
     Args:
         area_range (Tuple[float]): The candidate area scales range of
@@ -644,6 +767,9 @@ class RandomResizedCrop(RandomCrop):
                 to the next transform in pipeline.
         """
         _init_lazy_if_proper(results, self.lazy)
+        if 'keypoint' in results:
+            assert not self.lazy, ('Keypoint Augmentations are not compatible '
+                                   'with lazy == True')
 
         img_h, img_w = results['img_shape']
 
@@ -670,13 +796,16 @@ class RandomResizedCrop(RandomCrop):
         results['crop_quadruple'] = np.array(
             new_crop_quadruple, dtype=np.float32)
 
-        results['crop_bbox'] = np.array([left, top, right, bottom])
+        crop_bbox = np.array([left, top, right, bottom])
+        results['crop_bbox'] = crop_bbox
         results['img_shape'] = (new_h, new_w)
 
         if not self.lazy:
-            results['imgs'] = [
-                img[top:bottom, left:right] for img in results['imgs']
-            ]
+            if 'keypoint' in results:
+                results['keypoint'] = self._crop_kps(results['keypoint'],
+                                                     crop_bbox)
+            if 'imgs' in results:
+                results['imgs'] = self._crop_imgs(results['imgs'], crop_bbox)
         else:
             lazyop = results['lazy']
             if lazyop['flip']:
@@ -716,9 +845,11 @@ class MultiScaleCrop(RandomCrop):
     the base size, which is the minimal of image width and height. The scale
     level of w and h is controlled to be smaller than a certain value to
     prevent too large or small aspect ratio.
-    Required keys are "imgs", "img_shape", added or modified keys are "imgs",
-    "crop_bbox", "img_shape", "lazy" and "scales". Required keys in "lazy" are
-    "crop_bbox", added or modified key is "crop_bbox".
+
+    Required keys are "img_shape", "imgs" (optional), "keypoint" (optional),
+    added or modified keys are "imgs", "crop_bbox", "img_shape", "lazy" and
+    "scales". Required keys in "lazy" are "crop_bbox", added or modified key is
+    "crop_bbox".
 
     Args:
         input_size (int | tuple[int]): (w, h) of network input.
@@ -771,6 +902,9 @@ class MultiScaleCrop(RandomCrop):
                 to the next transform in pipeline.
         """
         _init_lazy_if_proper(results, self.lazy)
+        if 'keypoint' in results:
+            assert not self.lazy, ('Keypoint Augmentations are not compatible '
+                                   'with lazy == True')
 
         img_h, img_w = results['img_shape']
         base_size = min(img_h, img_w)
@@ -818,8 +952,9 @@ class MultiScaleCrop(RandomCrop):
 
         new_h, new_w = crop_h, crop_w
 
-        results['crop_bbox'] = np.array(
+        crop_bbox = np.array(
             [x_offset, y_offset, x_offset + new_w, y_offset + new_h])
+        results['crop_bbox'] = crop_bbox
         results['img_shape'] = (new_h, new_w)
         results['scales'] = self.scales
 
@@ -843,10 +978,11 @@ class MultiScaleCrop(RandomCrop):
             new_crop_quadruple, dtype=np.float32)
 
         if not self.lazy:
-            results['imgs'] = [
-                img[y_offset:y_offset + new_h, x_offset:x_offset + new_w]
-                for img in results['imgs']
-            ]
+            if 'keypoint' in results:
+                results['keypoint'] = self._crop_kps(results['keypoint'],
+                                                     crop_bbox)
+            if 'imgs' in results:
+                results['imgs'] = self._crop_imgs(results['imgs'], crop_bbox)
         else:
             lazyop = results['lazy']
             if lazyop['flip']:
@@ -884,10 +1020,10 @@ class MultiScaleCrop(RandomCrop):
 class Resize:
     """Resize images to a specific size.
 
-    Required keys are "imgs", "img_shape", "modality", added or modified
-    keys are "imgs", "img_shape", "keep_ratio", "scale_factor", "lazy",
-    "resize_size". Required keys in "lazy" is None, added or modified key is
-    "interpolation".
+    Required keys are "img_shape", "modality", "imgs" (optional), "keypoint"
+    (optional), added or modified keys are "imgs", "img_shape", "keep_ratio",
+    "scale_factor", "lazy", "resize_size". Required keys in "lazy" is None,
+    added or modified key is "interpolation".
 
     Args:
         scale (float | Tuple[int]): If keep_ratio is True, it serves as scaling
@@ -926,6 +1062,16 @@ class Resize:
         self.interpolation = interpolation
         self.lazy = lazy
 
+    def _resize_imgs(self, imgs, new_w, new_h):
+        return [
+            mmcv.imresize(
+                img, (new_w, new_h), interpolation=self.interpolation)
+            for img in imgs
+        ]
+
+    def _resize_kps(self, kps, scale_factor):
+        return kps * scale_factor
+
     def _box_resize(self, box, scale_factor):
         """Rescale the bounding boxes according to the scale_factor.
 
@@ -946,6 +1092,9 @@ class Resize:
         """
 
         _init_lazy_if_proper(results, self.lazy)
+        if 'keypoint' in results:
+            assert not self.lazy, ('Keypoint Augmentations are not compatible '
+                                   'with lazy == True')
 
         if 'scale_factor' not in results:
             results['scale_factor'] = np.array([1, 1], dtype=np.float32)
@@ -964,11 +1113,12 @@ class Resize:
         results['scale_factor'] = results['scale_factor'] * self.scale_factor
 
         if not self.lazy:
-            results['imgs'] = [
-                mmcv.imresize(
-                    img, (new_w, new_h), interpolation=self.interpolation)
-                for img in results['imgs']
-            ]
+            if 'imgs' in results:
+                results['imgs'] = self._resize_imgs(results['imgs'], new_w,
+                                                    new_h)
+            if 'keypoint' in results:
+                results['keypoint'] = self._resize_kps(results['keypoint'],
+                                                       self.scale_factor)
         else:
             lazyop = results['lazy']
             if lazyop['flip']:
@@ -1053,11 +1203,13 @@ class Flip:
 
     Reverse the order of elements in the given imgs with a specific direction.
     The shape of the imgs is preserved, but the elements are reordered.
-    Required keys are "imgs", "img_shape", "modality", added or modified
-    keys are "imgs", "lazy" and "flip_direction". Required keys in "lazy" is
-    None, added or modified key are "flip" and "flip_direction". The Flip
-    augmentation should be placed after any cropping / reshaping augmentations,
-    to make sure crop_quadruple is calculated properly.
+
+    Required keys are "img_shape", "modality", "imgs" (optional), "keypoint"
+    (optional), added or modified keys are "imgs", "keypoint", "lazy" and
+    "flip_direction". Required keys in "lazy" is None, added or modified key
+    are "flip" and "flip_direction". The Flip augmentation should be placed
+    after any cropping / reshaping augmentations, to make sure crop_quadruple
+    is calculated properly.
 
     Args:
         flip_ratio (float): Probability of implementing flip. Default: 0.5.
@@ -1065,6 +1217,10 @@ class Flip:
             "horizontal" | "vertical". Default: "horizontal".
         flip_label_map (Dict[int, int] | None): Transform the label of the
             flipped image with the specific label. Default: None.
+        left_kp (list[int]): Indexes of left keypoints, used to flip keypoints.
+            Default: None.
+        right_kp (list[ind]): Indexes of right keypoints, used to flip
+            keypoints. Default: None.
         lazy (bool): Determine whether to apply lazy operation. Default: False.
     """
     _directions = ['horizontal', 'vertical']
@@ -1073,6 +1229,8 @@ class Flip:
                  flip_ratio=0.5,
                  direction='horizontal',
                  flip_label_map=None,
+                 left_kp=None,
+                 right_kp=None,
                  lazy=False):
         if direction not in self._directions:
             raise ValueError(f'Direction {direction} is not supported. '
@@ -1080,8 +1238,32 @@ class Flip:
         self.flip_ratio = flip_ratio
         self.direction = direction
         self.flip_label_map = flip_label_map
+        self.left_kp = left_kp
+        self.right_kp = right_kp
         self.lazy = lazy
 
+    def _flip_imgs(self, imgs, modality):
+        _ = [mmcv.imflip_(img, self.direction) for img in imgs]
+        lt = len(imgs)
+        if modality == 'Flow':
+            # The 1st frame of each 2 frames is flow-x
+            for i in range(0, lt, 2):
+                imgs[i] = mmcv.iminvert(imgs[i])
+        return imgs
+
+    def _flip_kps(self, kps, kpscores, img_width):
+        kp_x = kps[..., 0]
+        kp_x[kp_x != 0] = img_width - kp_x[kp_x != 0]
+        new_order = list(range(kps.shape[2]))
+        if self.left_kp is not None and self.right_kp is not None:
+            for left, right in zip(self.left_kp, self.right_kp):
+                new_order[left] = right
+                new_order[right] = left
+        kps = kps[:, :, new_order]
+        if kpscores is not None:
+            kpscores = kpscores[:, :, new_order]
+        return kps, kpscores
+
     def _box_flip(self, box, img_width):
         """Flip the bounding boxes given the width of the image.
 
@@ -1102,6 +1284,13 @@ class Flip:
                 to the next transform in pipeline.
         """
         _init_lazy_if_proper(results, self.lazy)
+        if 'keypoint' in results:
+            assert not self.lazy, ('Keypoint Augmentations are not compatible '
+                                   'with lazy == True')
+            assert self.direction == 'horizontal', (
+                'Only horizontal flips are'
+                'supported for human keypoints')
+
         modality = results['modality']
         if modality == 'Flow':
             assert self.direction == 'horizontal'
@@ -1110,6 +1299,7 @@ class Flip:
 
         results['flip'] = flip
         results['flip_direction'] = self.direction
+        img_width = results['img_shape'][1]
 
         if self.flip_label_map is not None and flip:
             results['label'] = self.flip_label_map.get(results['label'],
@@ -1117,17 +1307,16 @@ class Flip:
 
         if not self.lazy:
             if flip:
-                for i, img in enumerate(results['imgs']):
-                    mmcv.imflip_(img, self.direction)
-                lt = len(results['imgs'])
-                for i in range(0, lt, 2):
-                    # flow with even indexes are x_flow, which need to be
-                    # inverted when doing horizontal flip
-                    if modality == 'Flow':
-                        results['imgs'][i] = mmcv.iminvert(results['imgs'][i])
-
-            else:
-                results['imgs'] = list(results['imgs'])
+                if 'imgs' in results:
+                    results['imgs'] = self._flip_imgs(results['imgs'],
+                                                      modality)
+                if 'keypoint' in results:
+                    kp = results['keypoint']
+                    kpscore = results.get('keypoint_score', None)
+                    kp, kpscore = self._flip_kps(kp, kpscore, img_width)
+                    results['keypoint'] = kp
+                    if 'keypoint_score' in results:
+                        results['keypoint_score'] = kpscore
         else:
             lazyop = results['lazy']
             if lazyop['flip']:
@@ -1426,9 +1615,10 @@ class ColorJitter:
 class CenterCrop(RandomCrop):
     """Crop the center area from images.
 
-    Required keys are "imgs", "img_shape", added or modified keys are "imgs",
-    "crop_bbox", "lazy" and "img_shape". Required keys in "lazy" is
-    "crop_bbox", added or modified key is "crop_bbox".
+    Required keys are "img_shape", "imgs" (optional), "keypoint" (optional),
+    added or modified keys are "imgs", "keypoint", "crop_bbox", "lazy" and
+    "img_shape". Required keys in "lazy" is "crop_bbox", added or modified key
+    is "crop_bbox".
 
     Args:
         crop_size (int | tuple[int]): (w, h) of crop size.
@@ -1450,6 +1640,9 @@ class CenterCrop(RandomCrop):
                 to the next transform in pipeline.
         """
         _init_lazy_if_proper(results, self.lazy)
+        if 'keypoint' in results:
+            assert not self.lazy, ('Keypoint Augmentations are not compatible '
+                                   'with lazy == True')
 
         img_h, img_w = results['img_shape']
         crop_w, crop_h = self.crop_size
@@ -1460,7 +1653,8 @@ class CenterCrop(RandomCrop):
         bottom = top + crop_h
         new_h, new_w = bottom - top, right - left
 
-        results['crop_bbox'] = np.array([left, top, right, bottom])
+        crop_bbox = np.array([left, top, right, bottom])
+        results['crop_bbox'] = crop_bbox
         results['img_shape'] = (new_h, new_w)
 
         if 'crop_quadruple' not in results:
@@ -1483,9 +1677,11 @@ class CenterCrop(RandomCrop):
             new_crop_quadruple, dtype=np.float32)
 
         if not self.lazy:
-            results['imgs'] = [
-                img[top:bottom, left:right] for img in results['imgs']
-            ]
+            if 'keypoint' in results:
+                results['keypoint'] = self._crop_kps(results['keypoint'],
+                                                     crop_bbox)
+            if 'imgs' in results:
+                results['imgs'] = self._crop_imgs(results['imgs'], crop_bbox)
         else:
             lazyop = results['lazy']
             if lazyop['flip']:
