@@ -328,6 +328,8 @@ class ResNet3d(nn.Module):
     Args:
         depth (int): Depth of resnet, from {18, 34, 50, 101, 152}.
         pretrained (str | None): Name of pretrained model.
+        stage_blocks (tuple | None): Set number of stages for each res layer.
+            Default: None.
         pretrained2d (bool): Whether to load pretrained 2D model.
             Default: True.
         in_channels (int): Channel num of input features. Default: 3.
@@ -390,6 +392,7 @@ class ResNet3d(nn.Module):
     def __init__(self,
                  depth,
                  pretrained,
+                 stage_blocks=None,
                  pretrained2d=True,
                  in_channels=3,
                  num_stages=4,
@@ -399,7 +402,9 @@ class ResNet3d(nn.Module):
                  temporal_strides=(1, 1, 1, 1),
                  dilations=(1, 1, 1, 1),
                  conv1_kernel=(5, 7, 7),
+                 conv1_stride_s=2,
                  conv1_stride_t=2,
+                 pool1_stride_s=2,
                  pool1_stride_t=2,
                  with_pool2=True,
                  style='pytorch',
@@ -425,6 +430,7 @@ class ResNet3d(nn.Module):
         self.base_channels = base_channels
         self.num_stages = num_stages
         assert 1 <= num_stages <= 4
+        self.stage_blocks = stage_blocks
         self.out_indices = out_indices
         assert max(out_indices) < num_stages
         self.spatial_strides = spatial_strides
@@ -432,8 +438,13 @@ class ResNet3d(nn.Module):
         self.dilations = dilations
         assert len(spatial_strides) == len(temporal_strides) == len(
             dilations) == num_stages
+        if self.stage_blocks is not None:
+            assert len(self.stage_blocks) == num_stages
+
         self.conv1_kernel = conv1_kernel
+        self.conv1_stride_s = conv1_stride_s
         self.conv1_stride_t = conv1_stride_t
+        self.pool1_stride_s = pool1_stride_s
         self.pool1_stride_t = pool1_stride_t
         self.with_pool2 = with_pool2
         self.style = style
@@ -449,7 +460,10 @@ class ResNet3d(nn.Module):
         self.zero_init_residual = zero_init_residual
 
         self.block, stage_blocks = self.arch_settings[depth]
-        self.stage_blocks = stage_blocks[:num_stages]
+
+        if self.stage_blocks is None:
+            self.stage_blocks = stage_blocks[:num_stages]
+
         self.inplanes = self.base_channels
 
         self.non_local_cfg = non_local_cfg
@@ -733,7 +747,8 @@ class ResNet3d(nn.Module):
             self.in_channels,
             self.base_channels,
             kernel_size=self.conv1_kernel,
-            stride=(self.conv1_stride_t, 2, 2),
+            stride=(self.conv1_stride_t, self.conv1_stride_s,
+                    self.conv1_stride_s),
             padding=tuple([(k - 1) // 2 for k in _triple(self.conv1_kernel)]),
             bias=False,
             conv_cfg=self.conv_cfg,
@@ -742,7 +757,8 @@ class ResNet3d(nn.Module):
 
         self.maxpool = nn.MaxPool3d(
             kernel_size=(1, 3, 3),
-            stride=(self.pool1_stride_t, 2, 2),
+            stride=(self.pool1_stride_t, self.pool1_stride_s,
+                    self.pool1_stride_s),
             padding=(0, 1, 1))
 
         self.pool2 = nn.MaxPool3d(kernel_size=(2, 1, 1), stride=(2, 1, 1))
