@@ -1,5 +1,6 @@
 from .conv2plus1d import Conv2plus1d
 from .conv_audio import ConvAudio
+from .lfb import LFB
 from .tam import TAM
 
-__all__ = ['Conv2plus1d', 'ConvAudio', 'TAM']
+__all__ = ['Conv2plus1d', 'ConvAudio', 'LFB', 'TAM']
