@@ -4,6 +4,7 @@ from .bbox_head import BBoxHeadAVA
 from .fbo_head import FBOHead
 from .i3d_head import I3DHead
 from .lfb_infer_head import LFBInferHead
+from .misc_head import ACRNHead
 from .roi_head import AVARoIHead
 from .slowfast_head import SlowFastHead
 from .ssn_head import SSNHead
@@ -16,5 +17,5 @@ from .x3d_head import X3DHead
 __all__ = [
     'TSNHead', 'I3DHead', 'BaseHead', 'TSMHead', 'SlowFastHead', 'SSNHead',
     'TPNHead', 'AudioTSNHead', 'X3DHead', 'BBoxHeadAVA', 'AVARoIHead',
-    'FBOHead', 'LFBInferHead', 'TRNHead'
+    'FBOHead', 'LFBInferHead', 'TRNHead', 'ACRNHead'
 ]
