@@ -22,14 +22,16 @@ class BaseRecognizer(nn.Module, metaclass=ABCMeta):
 
     Args:
         backbone (dict): Backbone modules to extract feature.
-        cls_head (dict): Classification head to process feature.
+        cls_head (dict | None): Classification head to process feature.
+            Default: None.
+        neck (dict | None): Neck for feature fusion. Default: None.
         train_cfg (dict | None): Config for training. Default: None.
         test_cfg (dict | None): Config for testing. Default: None.
     """
 
     def __init__(self,
                  backbone,
-                 cls_head,
+                 cls_head=None,
                  neck=None,
                  train_cfg=None,
                  test_cfg=None):
@@ -63,7 +65,8 @@ class BaseRecognizer(nn.Module, metaclass=ABCMeta):
 
         if neck is not None:
             self.neck = builder.build_neck(neck)
-        self.cls_head = builder.build_head(cls_head)
+
+        self.cls_head = builder.build_head(cls_head) if cls_head else None
 
         self.train_cfg = train_cfg
         self.test_cfg = test_cfg
@@ -80,6 +83,11 @@ class BaseRecognizer(nn.Module, metaclass=ABCMeta):
             self.max_testing_views = test_cfg['max_testing_views']
             assert isinstance(self.max_testing_views, int)
 
+        if test_cfg is not None and 'feature_extraction' in test_cfg:
+            self.feature_extraction = test_cfg['feature_extraction']
+        else:
+            self.feature_extraction = False
+
         # mini-batch blending, e.g. mixup, cutmix, etc.
         self.blending = None
         if train_cfg is not None and 'blending' in train_cfg:
@@ -93,9 +101,14 @@ class BaseRecognizer(nn.Module, metaclass=ABCMeta):
 
     @property
     def with_neck(self):
-        """bool: whether the detector has a neck"""
+        """bool: whether the recognizer has a neck"""
         return hasattr(self, 'neck') and self.neck is not None
 
+    @property
+    def with_cls_head(self):
+        """bool: whether the recognizer has a cls_head"""
+        return hasattr(self, 'cls_head') and self.cls_head is not None
+
     def init_weights(self):
         """Initialize the model network weights."""
         if self.backbone_from in ['mmcls', 'mmaction2']:
@@ -109,7 +122,8 @@ class BaseRecognizer(nn.Module, metaclass=ABCMeta):
             raise NotImplementedError('Unsupported backbone source '
                                       f'{self.backbone_from}!')
 
-        self.cls_head.init_weights()
+        if self.with_cls_head:
+            self.cls_head.init_weights()
         if self.with_neck:
             self.neck.init_weights()
 
